#!/usr/bin/env python

# $Id: database.py 82 2011-02-18 07:44:15Z kkumer $ 

"""
Contacting MySQL database, both via MySQLdb and via SQLObject.
"""


import MySQLdb, re, string, cPickle
from sqlobject import *

#--------------------------------------------------
# Connection to database via MySQLdb
#-------------------------------------------------- 


def getCursor():
    db = MySQLdb.Connect(user=cfg['DB_USER'], db=cfg['DB_NAME'], passwd=cfg['DB_PASSWD'], 
            charset="utf8", use_unicode=True)
    dbc = db.cursor()
    dbc.execute('SET NAMES utf8;')
    dbc.execute('SET CHARACTER SET utf8;')
    dbc.execute('SET character_set_connection=utf8;')
    return dbc

def releaseCursor(cursor):
    cursor.connection.close()


#----------------------------------------------------------------
# Fields - array holding instances of FieldDataDescription, each
# instance corresponding to:
#     1) line in register.cf
#     2) row in database table 'People'
#     3) item in HTML form sending data to CGI script register.py
#----------------------------------------------------------------

class FieldDataDescription:
    """Nothing fancy. Just to be able tu use attributes instead of
       tuple indices"""
    def __init__(self, line):
        """ Initialization form line of configuration file """
        self.data = tuple(string.split(line)[1:])
        self.name = self.data[0]
        self.htype = self.data[1]
        self.obligatory = self.data[3]
        self.default = self.data[4]
        self.description = self.data[5] 

def FieldList():
    """ Parses configuration file and returns a list of 
    FieldDataDescription instances. Each item in this list
    corresponds to one datum to be sent by HTML form and to one column 
    in 'people' table in database.
    """

    fields = []
    file = open(cfg['REGISTER_CONF_FILE'], "r")
    line = file.readline()
    while line:
        # only lines starting with 'FIELD' are parsed
        if re.match(r'FIELD', line):
            fields.append(FieldDataDescription(line))
        line = file.readline()
    
    return fields


#--------------------------------------------------
# Connection to database via SQLObject
#-------------------------------------------------- 

#--------------------------------------------------------------
# We need SQLObject subclass 'People' that represents
# our table. See SQLObject documentation. This class is created
# dynamically i.e. wia python exec command by creating string
# with lines of type:
#    <fieldname> = StringCol(dbName='<fieldname>')
# using the above defined array 'Fields'
# TODO: There should be more elegant ways to do that than via exec!
#--------------------------------------------------------------


def dbconnect():
    """Connect to database and return SQLObject subclass 'People'."""

    connection_string = 'mysql://%s:%s@localhost/%s?charset=utf8' %\
            (cfg['DB_USER'], cfg['DB_PASSWD'], cfg['DB_NAME'])
    connection = connectionForURI(connection_string)
    sqlhub.processConnection = connection

    Fields = FieldList()
    cmd = 'class People(SQLObject):\n'
    for field in Fields:
        cmd += '    ' + field.name + ' = StringCol(dbName="'\
              + field.name + '")\n'  
    exec cmd
    return People



class Author:
    """Just a "struct" for convenient access of author fields."""

    def __init__(self, tpl):
        self.name = tpl[0]
        self.surname = tpl[1]
        self.email = tpl[2]
        self.instid = tpl[3]

def put(cursor, authorid, abstract):
    """Creates new abstract in table 'abstracts'.
       put(<cursor object>, integer, dictionary) -> None"""

    qry = 'INSERT INTO abstracts (authorid, body) VALUES (%u, "%s");' % \
        (authorid, MySQLdb.escape_string(cPickle.dumps(abstract)))
    cursor.execute(qry)

def get(cursor, authorid):
    """Gets abstract corresponding to id.
       get(<cursor object>, integer) -> dictionary"""

    qry = 'SELECT body FROM abstracts WHERE authorid = %u' % authorid
    cursor.execute(qry)
    res = cursor.fetchall()
    if len(res) <> 1:
        raise RuntimeError, "Nonexistant (or multiple) authorid"
    abstract = cPickle.loads(res[0][0])
    return abstract

def update(cursor, authorid, abstract):
    """Updates abstract corresponding to id in table 'abstracts'.
       update(<cursor object>, integer, dictionary) -> None"""

    qry = 'UPDATE abstracts SET body="%s" WHERE authorid=%u;' % \
        (MySQLdb.escape_string(cPickle.dumps(abstract)), authorid)
    cursor.execute(qry)

