#!/usr/bin/env python

# $Id: web.py 82 2011-02-18 07:44:15Z kkumer $

"""
Functions that create actual statical HTML pages and help
in creating dynamical ones.

Functions that pretty-print (ascii or HTML) results of SQLObject 
database selects.
"""

import re
from database import *
from HTMLgen import *


def publish(pages):
    template = open(cfg['TEMPLATE_FILE'], 'r').read()
    for page in pages:
        dict = {}
        dict['TITLE'] = page[1]
        dict['UPDATE'] = page[2]
        dict['CONTENT'] = page[3]
        abspath = cfg['HTML_ROOT'] + page[0]
        absdir = os.path.dirname(abspath)
        # If dir doesn't exist we create it:
        if not os.path.isdir(absdir): 
            if cfg['DEBUG']: print 'Creating directory %s' % absdir
            os.makedirs(absdir)
        if cfg['DEBUG']: print 'Deploying "%s" to %s' % (page[1], page[0])
        open(cfg['HTML_ROOT'] + page[0], 'w').write(template % dict)
    print "Done."

def extractHF():
    """Returns header and footer of HTML template for CGI scripts
       extractHF() -> string, string
    """
    template = open(cfg['TEMPLATE_FILE'], 'r').read()
    head = re.compile(r'<!-- BEGIN head .* END head -->', re.DOTALL).search(template).group()
    foot = re.compile(r'<!-- BEGIN foot .* END foot -->', re.DOTALL).search(template).group()
    return head, foot


def maxlen(rows, item):
    """Maximal length (in characters) of item
       maxlen(SelectResults, str) -> int
    """
    if rows.count() == 0:
        return "======  Nothing found!  ======"
    l = 1
    for row in rows:
        newl = len(str(eval('row.' + item)))
        if newl > l: l = newl
    if l > 30: l = 20
    return l

def asciiTable(rows, *items):
    """ Formatted ascii table with SELECT results.
        aT, asciiTable(SelectResults, *str) -> str 
    """
    if rows.count() == 0:
        return "======  Nothing found!  ======"
    columns = list(items)
    s = ""
    tot = 1
    for col in columns:
        max = maxlen(rows, col)
        s += '| %' + str(max) + '.' + str(max) + 's '
        tot += max + 3
    s += '|\n'
    T = tot*"=" + "\n"
    for row in rows:
        T += s % tuple(
             [eval('row.' + colname) for colname in columns])
    T += tot*"=" + "\n"
    return T

aT = asciiTable

def asciiRecord(People, id):
    """Complete record (People table row) of id formatted as ascii table.
       aR, asciiRecord(int) -> str
    """
    s = ('\n' + 32*'=' + '[ %4d ]' + 32*'=' + '\n') % id
    for field in FieldList():
        val = People.get(id).__getattribute__(field.name)
        s += ' %12s =>  %.60s \n' % (field.name, val)
    s += 72*'=' + '\n'
    return s

aR = asciiRecord

def htmlTable(rows, *items):
    """ HTML table with SELECT results.
        hT, htmlTable(SelectResults, *str) -> str 
    """
    if rows.count() == 0:
        return "<BR><P><B> Nothing found! </B><BR><P>"
    columns = list(items)
    T = Table()
    T.heading = columns
    T.body = []
    for row in rows:
        T.body.append(
             [eval('row.' + colname) for colname in columns])
    return T

hT = htmlTable

def htmlTableLinked(rows, *items):
    """ HTML table with SELECT results. ID's linked to htmlRecord()
        hTL, htmlTableLinked(SelectResults, *str) -> str 
    """
    if rows.count() == 0:
        return "<BR><P><B> Nothing found! </B><BR><P>"
    columns = list(items)
    T = Table()
    T.heading = ['id'] + columns
    T.body = []
    for row in rows:
        T.body.append([Href(cfg['CGI_LINK'] + 'person.py?id=%i' % int(row.id), 
                        str(row.id))] +
                [eval('row.' + colname) for colname in columns])
    return T

hTL = htmlTableLinked

def htmlTableAbstracts(rows, *items):
    """ HTML table with SELECT results. ID's linked to htmlRecord()
        This is conf-specific.
        hTA, htmlTableAbstracts(SelectResults, *str) -> str 
    """
    columns = list(items)
    T = Table()
    T.colspan = (len(columns)+1)*[1] + [5]
    T.heading = ['id'] + columns + ['Sazetak']
    T.body = []
    for row in rows:
        T.body.append([Href(cfg['CGI_LINK'] + 'home.py?pwd=' +
             makepwd(int(row.id)), str(row.id))] +
             [eval('row.' + colname) for colname in columns]+
             hisAbstract(row))
    return T

hTA = htmlTableAbstracts

def htmlRecord(People, id):
    """Complete record (table row) of id formatted as HTML table.
       hR, htmlRecord(SQLObject, int) -> str
    """
    T = Table(str(id))
    T.heading = ['column', 'value']
    T.body = []
    for field in FieldList():
        val = People.get(id).__getattribute__(field.name)
        T.body.append([field.name, val])
    return T

hR = htmlRecord

def htmlRecordEdit(People, id):
    """Complete record (table row) of id formatted as HTML form.
       Editing and submiting this form updates database.
       Works only for text items at the moment.
       hR, htmlRecord(SQLObject, int) -> str
    """
    f = Form(cgi=cfg['CGI_LINK']+"register.py",
         submit=Input(type="submit", value="Submit"),
         reset=Input(type="reset", value="Reset"))
    T = Table(str(id))
    T.heading = ['column', 'value']
    T.body = []
    for field in FieldList():
        if field.htype == 'radio': break
        # otherwise it's text / varchar
        val = People.get(id).__getattribute__(field.name)
        inp = Input(type="text",
                    name=field.name,
                    value=val,
                    size=60)
        T.body.append([field.name, inp])
    f.append(T)
    return f

hRE = htmlRecordEdit

def asciiSearchAll(People, str):
    """Searches all fields in table for substring
       aSA, asciiSearchAll(string) -> printout
    """
    aux = 'People.select(OR('
    for field in FieldList():
        aux += 'People.q.' + field.name + '.contains("' + str + '"),'
    aux = aux[:-1] + '))'
    sel = eval(aux)
    print aT(sel, 'id', 'name', 'surname')

aSA = asciiSearchAll

def asciiSearchName(People, str):
    """Searches 'name' and 'surname' fields in table for substring
       aSN, asciiSearchName(string) -> printout
    """
    aux = 'People.select(OR('
    for item in ['name', 'surname']:
        aux += 'People.q.' + item + '.contains("' + str + '"),'
    aux = aux[:-1] + '))'
    sel = eval(aux)
    print aT(sel, 'id', 'name', 'surname')

aSN = asciiSearchName

def htmlSearchAll(People, str):
    """Searches all fields in table for substring
       aSA, asciiSearchAll(string) -> html code
    """
    aux = 'People.select(OR('
    for field in FieldList():
        aux += 'People.q.' + field.name + '.contains("' + str + '"),'
    aux = aux[:-1] + '))'
    sel = eval(aux)
    return hTL(sel, 'name', 'surname')

aSA = asciiSearchAll

def htmlSearchName(People, str):
    """Searches 'name' and 'surname' fields in table for substring
       aSN, asciiSearchName(string) -> html code
    """
    aux = 'People.select(OR('
    for item in ['name', 'surname']:
        aux += 'People.q.' + item + '.contains("' + str + '"),'
    aux = aux[:-1] + '))'
    sel = eval(aux)
    return hTL(sel, 'name', 'surname')

hSN = htmlSearchName
